@@ -1,21 +1,24 @@
 import json
 
+from octue.utils.decoders import OctueJSONDecoder
 from octue.utils.encoders import OctueJSONEncoder
 
 
 class Serialisable:
     """Mixin class to make resources serialisable to JSON.
 
-    Objects must have a `.logger` and a `.id` property
+    The `logger` field is always excluded from serialisation if it is present.
     """
 
     _SERIALISE_FIELDS = None
     _EXCLUDE_SERIALISE_FIELDS = ("logger",)
 
     def __init__(self, *args, **kwargs):
-        """Constructor for serialisable mixin"""
         super().__init__(*args, **kwargs)
 
+        if "logger" not in self._EXCLUDE_SERIALISE_FIELDS:
+            self._EXCLUDE_SERIALISE_FIELDS = (*self._EXCLUDE_SERIALISE_FIELDS, "logger")
+
     @classmethod
     def deserialise(cls, serialised_object, from_string=False):
         """Deserialise the given JSON-serialised object.
@@ -25,7 +28,7 @@ class Serialisable:
         :return any:
         """
         if from_string:
-            serialised_object = json.loads(serialised_object)
+            serialised_object = json.loads(serialised_object, cls=OctueJSONDecoder)
 
         return cls(**serialised_object)
 
@@ -35,7 +38,6 @@ class Serialisable:
         :parameter str file_name: file to write to, including relative or absolute path and .json extension
         :return None:
         """
-        self.logger.debug("Writing %s %s to file %s", self.__class__.__name__, self.id, file_name)
         with open(file_name, "w") as fp:
             fp.write(self.serialise(**kwargs, to_string=True))
 
@@ -62,8 +64,6 @@ class Serialisable:
         :return: json string or dict containing a serialised/primitive version of the resource.
         :rtype: str, dict
         """
-        self.logger.debug("Serialising %s %s", self.__class__.__name__, self.id)
-
         # Get all non-private and non-protected attributes except those excluded specifically
         names_of_attributes_to_serialise = self._SERIALISE_FIELDS or (
             field_name
@@ -78,8 +78,12 @@ class Serialisable:
         self_as_primitive = {}
         for name in names_of_attributes_to_serialise:
             attribute = getattr(self, name, None)
+
             # Serialise sets as sorted list (JSON doesn't support sets).
-            self_as_primitive[name] = sorted(attribute) if isinstance(attribute, set) else attribute
+            if isinstance(attribute, set):
+                self_as_primitive[name] = sorted(attribute)
+            else:
+                self_as_primitive[name] = attribute
 
         # TODO this conversion backward-and-forward is very inefficient but allows us to use the same encoder for
         #  converting the object to a dict as to strings, which ensures that nested attributes are also cast to
@@ -91,4 +95,4 @@ class Serialisable:
         if to_string:
             return string
 
-        return json.loads(string)
+        return json.loads(string, cls=OctueJSONDecoder)
