@@ -1,5 +1,6 @@
 from unittest.mock import patch
 import google.api_core.exceptions
+from google.pubsub_v1 import SubscriberClient
 
 from octue.cloud.pub_sub.service import Service
 from octue.cloud.pub_sub.subscription import SEVEN_DAYS, THIRTY_ONE_DAYS, Subscription
@@ -7,20 +8,27 @@ from octue.cloud.pub_sub.topic import Topic
 from octue.resources.service_backends import GCPPubSubBackend
 from tests import TEST_PROJECT_NAME
 from tests.base import BaseTestCase
+from tests.cloud.pub_sub.mocks import MockSubscriber
 
 
 class TestSubscription(BaseTestCase):
 
     service = Service(backend=GCPPubSubBackend(project_name="my-project"))
     topic = Topic(name="world", namespace="hello", service=service)
-    subscription = Subscription(name="world", topic=topic, namespace="hello", service=service)
+    subscription = Subscription(
+        name="world", topic=topic, namespace="hello", project_name=TEST_PROJECT_NAME, subscriber=MockSubscriber()
+    )
 
     def test_namespace_only_in_name_once(self):
         """Test that the subscription's namespace only appears in its name once, even if it is repeated."""
         self.assertEqual(self.subscription.name, "hello.world")
 
         subscription_with_repeated_namespace = Subscription(
-            name="hello.world", topic=self.topic, namespace="hello", service=self.service
+            name="hello.world",
+            topic=self.topic,
+            namespace="hello",
+            project_name=TEST_PROJECT_NAME,
+            subscriber=MockSubscriber(),
         )
 
         self.assertEqual(subscription_with_repeated_namespace.name, "hello.world")
@@ -34,7 +42,7 @@ class TestSubscription(BaseTestCase):
         `False`.
         """
         with patch(
-            "octue.cloud.pub_sub.service.pubsub_v1.SubscriberClient.create_subscription",
+            "tests.cloud.pub_sub.mocks.MockSubscriber.create_subscription",
             side_effect=google.api_core.exceptions.AlreadyExists(""),
         ):
             with self.assertRaises(google.api_core.exceptions.AlreadyExists):
@@ -45,7 +53,7 @@ class TestSubscription(BaseTestCase):
         error.
         """
         with patch(
-            "octue.cloud.pub_sub.service.pubsub_v1.SubscriberClient.create_subscription",
+            "tests.cloud.pub_sub.mocks.MockSubscriber.create_subscription",
             side_effect=google.api_core.exceptions.AlreadyExists(""),
         ):
             self.subscription.create(allow_existing=True)
@@ -54,7 +62,13 @@ class TestSubscription(BaseTestCase):
         """Test that creating a subscription works properly."""
         service = Service(backend=GCPPubSubBackend(project_name=TEST_PROJECT_NAME))
         topic = Topic(name="my-topic", namespace="tests", service=service)
-        subscription = Subscription(name="world", topic=topic, namespace="hello", service=service)
+        subscription = Subscription(
+            name="world",
+            topic=topic,
+            namespace="hello",
+            project_name=TEST_PROJECT_NAME,
+            subscriber=SubscriberClient(credentials=service._credentials),
+        )
 
         try:
             topic.create(allow_existing=True)
